# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import, unicode_literals
from future.utils import PY3
from past.builtins import basestring
from builtins import *  # noqa

import warnings

if PY3:
    from urllib.parse import parse_qsl, urlparse
else:
    from urlparse import parse_qsl, urlparse

import gmusicapi
from gmusicapi.clients.shared import _Base
from gmusicapi.exceptions import GmusicapiWarning
from gmusicapi.protocol import webclient
from gmusicapi.utils import utils
import gmusicapi.session


class Webclient(_Base):
    """Allows library management and streaming by posing as the
    music.google.com webclient.

    Uploading is not supported by this client (use the :class:`Musicmanager`
    to upload).

    Any methods in this class that are duplicated by
    the :class:`Mobileclient` are deprecated, and will generate a
    warning at runtime.

    The following methods are *not* deprecated:

        * :func:`get_shared_playlist_info`
        * :func:`get_song_download_info`
        * :func:`get_stream_urls`
        * :func:`get_stream_audio`
        * :func:`report_incorrect_match`
        * :func:`upload_album_art`
    """

    _session_class = gmusicapi.session.Webclient

    def __init__(self, debug_logging=True, validate=True, verify_ssl=True):
        warnings.warn(
            "Webclient functionality is not tested nor well supported. "
            "Use Mobileclient or Musicmanager if possible.",
            GmusicapiWarning
        )

        super(Webclient, self).__init__(self.__class__.__name__,
                                        debug_logging,
                                        validate,
                                        verify_ssl)

    def login(self, email, password):
        """Authenticates the webclient.
        Returns ``True`` on success, ``False`` on failure.

        :param email: eg ``'test@gmail.com'`` or just ``'test'``.
        :param password: the account's password.
          This is not stored locally, and is sent securely over SSL.
          App-specific passwords are not supported on the webclient.

        Users who don't use two-factor auth will likely need to enable
        `less secure login <https://www.google.com/settings/security/lesssecureapps>`__.
        If this is needed, a warning will be logged during login (which will print to stderr
        in the default logging configuration).
        """

        if not self.session.login(email, password):
            self.logger.info("failed to authenticate")
            return False

        self.logger.info("authenticated")

        return True

    def logout(self):
        return super(Webclient, self).logout()

    def get_shared_playlist_info(self, share_token):
        """
        Returns a dictionary with four keys: author, description, num_tracks, and title.

        :param share_token: from ``playlist['shareToken']``, or a playlist share
          url (``https://play.google.com/music/playlist/<token>``).

          Note that tokens from urls will need to be url-decoded,
          eg ``AM...%3D%3D`` becomes ``AM...==``.
        """

        res = self._make_call(webclient.GetSharedPlaylist, '', share_token)
        num_tracks = len(res[1][0])
        md = res[1][1]

        return {
            u'author': md[8],
            u'description': md[7],
            u'num_tracks': num_tracks,
            u'title': md[1],
        }

    @utils.enforce_id_param
    def get_song_download_info(self, song_id):
        """Returns a tuple: ``('<url>', <download count>)``.

        :param song_id: a single song id.

        ``url`` will be ``None`` if the download limit is exceeded.

        GM allows 2 downloads per song. The download count may not always be accurate,
        and the 2 download limit seems to be loosely enforced.

        This call alone does not count towards a download -
        the count is incremented when ``url`` is retrieved.
        """

        # TODO the protocol expects a list of songs - could extend with accept_singleton
        info = self._make_call(webclient.GetDownloadInfo, [song_id])
        url = info.get('url')

        return (url, info["downloadCounts"][song_id])

    @utils.enforce_id_param
    def get_stream_urls(self, song_id):
        """Returns a list of urls that point to a streamable version of this song.

        If you just need the audio and are ok with gmusicapi doing the download,
        consider using :func:`get_stream_audio` instead.
        This abstracts away the differences between different kinds of tracks:

            * normal tracks return a single url
            * All Access tracks return multiple urls, which must be combined

        :param song_id: a single song id.

        While acquiring the urls requires authentication, retreiving the
        contents does not.

        However, there are limitations on how the stream urls can be used:

            * the urls expire after a minute
            * only one IP can be streaming music at once.
              Other attempts will get an http 403 with
              ``X-Rejected-Reason: ANOTHER_STREAM_BEING_PLAYED``.

        *This is only intended for streaming*. The streamed audio does not contain metadata.
        Use :func:`get_song_download_info` or :func:`Musicmanager.download_song
        <gmusicapi.clients.Musicmanager.download_song>`
        to download files with metadata.
        """

        res = self._make_call(webclient.GetStreamUrl, song_id)

        try:
            return [res['url']]
        except KeyError:
            return res['urls']

    @utils.enforce_id_param
    def get_stream_audio(self, song_id, use_range_header=None):
        """Returns a bytestring containing mp3 audio for this song.

        :param song_id: a single song id
        :param use_range_header: in some cases, an HTTP range header can be
          used to save some bandwidth.
          However, there's no guarantee that the server will respect it,
          meaning that the client may get back an unexpected response when
          using it.

          There are three possible values for this argument:
              * None: (default) send header; fix response locally on problems
              * True: send header; raise IOError on problems
              * False: do not send header
        """

        urls = self.get_stream_urls(song_id)

        # TODO shouldn't session.send be used throughout?

        if len(urls) == 1:
            return self.session._rsession.get(urls[0]).content

        # AA tracks are separated into multiple files.
        # the url contains the range of each file to be used.

        range_pairs = [[int(s) for s in val.split('-')]
                       for url in urls
                       for key, val in parse_qsl(urlparse(url)[4])
                       if key == 'range']

        stream_pieces = bytearray()
        prev_end = 0
        headers = None

        for url, (start, end) in zip(urls, range_pairs):
            if use_range_header or use_range_header is None:
                headers = {'Range': 'bytes=' + str(prev_end - start) + '-'}

            audio = self.session._rsession.get(url, headers=headers).content

            if end - prev_end != len(audio) - 1:
                # content length is not in the right range

                if use_range_header:
                    # the user didn't want automatic response fixup
                    raise IOError('use_range_header is True but the response'
                                  ' was not the correct content length.'
                                  ' This might be caused by a (poorly-written) http proxy.')

                # trim to the proper range
                audio = audio[prev_end - start:]

            stream_pieces.extend(audio)

            prev_end = end + 1

        return bytes(stream_pieces)

    @utils.accept_singleton(basestring)
    @utils.enforce_ids_param
    @utils.empty_arg_shortcircuit
    def report_incorrect_match(self, song_ids):
        """Equivalent to the 'Fix Incorrect Match' button, this requests re-uploading of songs.
        Returns the song_ids provided.

        :param song_ids: a list of song ids to report, or a single song id.

        Note that if you uploaded a song through gmusicapi, it won't be reuploaded
        automatically - this currently only works for songs uploaded with the Music Manager.
        See issue `#89 <https://github.com/simon-weber/gmusicapi/issues/89>`__.

        This should only be used on matched tracks (``song['type'] == 6``).
        """

        self._make_call(webclient.ReportBadSongMatch, song_ids)

        return song_ids

    @utils.accept_singleton(basestring)
    @utils.enforce_ids_param
    @utils.empty_arg_shortcircuit
    def upload_album_art(self, song_ids, image_filepath):
        """Uploads an image and sets it as the album art for songs.
        Returns a url to the image on Google's servers.

        :param song_ids: a list of song ids, or a single song id.
        :param image_filepath: filepath of the art to use. jpg and png are known to work.

        This function will *always* upload the provided image, even if it's already uploaded.
        If the art is already uploaded and set for another song, copy over the
        value of the ``'albumArtUrl'`` key using :func:`Mobileclient.change_song_metadata` instead.
        """

        res = self._make_call(webclient.UploadImage, image_filepath)
        url = res['imageUrl']

        song_dicts = [dict((('id', id), ('albumArtUrl', url))) for id in song_ids]

        self._make_call(webclient.ChangeSongMetadata, song_dicts)

        return url

    @utils.accept_singleton(dict)
    @utils.empty_arg_shortcircuit
    def change_song_metadata(self, songs):
        """Changes metadata of songs.

        Returns a list of the song ids changed.

        :param songs: a list of song dictionaries, each dictionary must contain valid song 'id'

        The following fields are supported: title, album, albumArtist, artist
        """

        self._make_call(webclient.ChangeSongMetadata, songs)

        return list(song['id'] for song in songs)

    # deprecated methods follow:

    @utils.deprecated('prefer Mobileclient.create_playlist')
    def create_playlist(self, name, description=None, public=False):
        """
        Creates a playlist and returns its id.

        :param name: the name of the playlist.
        :param description: (optional) the description of the playlist.
        :param public: if True and the user has All Access, create a shared playlist.
        """
        res = self._make_call(webclient.CreatePlaylist, name, description, public)

        return res[1][0]

    @utils.deprecated('prefer Mobileclient.get_registered_devices')
    def get_registered_devices(self):
        """
        Returns a list of dictionaries representing devices associated with the account.

        Performing the :class:`Musicmanager` OAuth flow will register a device
        of type 1.

        Installing the Google Music app on an android or ios device
        and logging into it will register a device of type 2 or 3,
        which is used for streaming with the :class:`Mobileclient`.

        Here is an example response::

            [
              {
                u'deviceType': 1,  # laptop/desktop
                u'id': u'00:11:22:33:AA:BB',
                u'lastAccessedFormatted': u'May 24, 2015',
                u'lastAccessedTimeMillis': 1432468588200,  # utc-millisecond
                u'lastEventTimeMillis': 1434211605335,
                u'name': u'my computer'},
              },
              {
                u'deviceType': 2,  # android device
                u'carrier': u'Google',
                u'id': u'0x00112233aabbccdd',  # remove 0x when streaming
                u'lastAccessedFormatted': u'September 19, 2015',
                u'lastAccessedTimeMillis': 1442706069906,
                u'lastEventTimeMillis': 1435271137193,
                u'manufacturer': u'Asus',
                u'model': u'Nexus 7',
                u'name': u'my nexus 7'
              },
              {
                u'deviceType': 3,  # ios device
                u'id': u'ios:01234567-0123-0123-0123-0123456789AB',
                u'lastAccessedFormatted': u'June 25, 2015',
                u'lastAccessedTimeMillis': 1435271588780,
                u'lastEventTimeMillis': 1435271442417,
                u'name': u'my iphone'
              }
            ]
        """

        # TODO sessionid stuff
        res = self._make_call(webclient.GetSettings, '')
        return res['settings']['uploadDevice']

    @utils.accept_singleton(basestring)
    @utils.enforce_ids_param
    @utils.empty_arg_shortcircuit
    @utils.deprecated('prefer Mobileclient.delete_songs')
    def delete_songs(self, song_ids):
        """**Deprecated**: prefer :func:`Mobileclient.delete_songs`.

        Deletes songs from the entire library. Returns a list of deleted song ids.

        :param song_ids: a list of song ids, or a single song id.
        """

        res = self._make_call(webclient.DeleteSongs, song_ids)

        return res['deleteIds']

    @utils.accept_singleton(basestring, 2)
    @utils.enforce_ids_param(2)
    @utils.enforce_id_param
    @utils.empty_arg_shortcircuit(position=2)
    @utils.deprecated('prefer Mobileclient.add_songs_to_playlist')
    def add_songs_to_playlist(self, playlist_id, song_ids):
        """**Deprecated**: prefer :func:`Mobileclient.add_songs_to_playlist`.

        Appends songs to a playlist.
        Returns a list of (song id, playlistEntryId) tuples that were added.

        :param playlist_id: id of the playlist to add to.
        :param song_ids: a list of song ids, or a single song id.

        Playlists have a maximum size of 1000 songs.
        """

        res = self._make_call(webclient.AddToPlaylist, playlist_id, song_ids)
        new_entries = res['songIds']

        return [(e['songId'], e['playlistEntryId']) for e in new_entries]

    @utils.accept_singleton(basestring, 2)
    @utils.enforce_ids_param(2)
    @utils.enforce_id_param
    @utils.empty_arg_shortcircuit(position=2)
    @utils.deprecated('prefer Mobileclient.remove_entries_from_playlist')
    def remove_songs_from_playlist(self, playlist_id, sids_to_match):
        """**Deprecated**: prefer :func:`Mobileclient.remove_entries_from_playlist`.

        Removes all copies of the given song ids from a playlist.
        Returns a list of removed (sid, eid) pairs.

        :param playlist_id: id of the playlist to remove songs from.
        :param sids_to_match: a list of song ids to match, or a single song id.

        This does *not always* the inverse of a call to :func:`add_songs_to_playlist`,
        since multiple copies of the same song are removed.
        """

        playlist_tracks = self.get_playlist_songs(playlist_id)
        sid_set = set(sids_to_match)

        matching_eids = [t["playlistEntryId"]
                         for t in playlist_tracks
                         if t["id"] in sid_set]

        if matching_eids:
            # Call returns "sid_eid" strings.
            sid_eids = self._remove_entries_from_playlist(playlist_id,
                                                          matching_eids)
            return [s.split("_") for s in sid_eids]
        else:
            return []

    @utils.accept_singleton(basestring, 2)
    @utils.empty_arg_shortcircuit(position=2)
    def _remove_entries_from_playlist(self, playlist_id, entry_ids_to_remove):
        """Removes entries from a playlist. Returns a list of removed "sid_eid" strings.

        :param playlist_id: the playlist to be modified.
        :param entry_ids: a list of entry ids, or a single entry id.
        """

        # GM requires the song ids in the call as well; find them.
        playlist_tracks = self.get_playlist_songs(playlist_id)
        remove_eid_set = set(entry_ids_to_remove)

        e_s_id_pairs = [(t["id"], t["playlistEntryId"])
                        for t in playlist_tracks
                        if t["playlistEntryId"] in remove_eid_set]

        num_not_found = len(entry_ids_to_remove) - len(e_s_id_pairs)
        if num_not_found > 0:
            self.logger.warning("when removing, %d entry ids could not be found in playlist id %s",
                                num_not_found, playlist_id)

        # Unzip the pairs.
        sids, eids = list(zip(*e_s_id_pairs))

        res = self._make_call(webclient.DeleteSongs, sids, playlist_id, eids)

        return res['deleteIds']



# -*- coding: utf-8 -*-

"""
Tools to handle Google's ridiculous interchange format.
"""
from __future__ import print_function, division, absolute_import, unicode_literals
from builtins import *  # noqa

from io import StringIO
from tokenize import generate_tokens

import json


def to_json(s):
    """Return a valid json string, given a jsarray string.

    :param s: string of jsarray data
    """
    out = []

    for t in generate_tokens(StringIO(s).readline):
        if out and any(((',' == t[1] == out[-1]),  # double comma
                        (out[-1] == '[' and t[1] == ','),  # comma opening array
                        )):
            out.append('null')

        out.append(t[1])

    return ''.join(out)


def loads(s):
    return json.loads(to_json(s))



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: download.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='download.proto',
  package='wireless_android_skyjam',
  syntax='proto2',
  serialized_pb=_b('\n\x0e\x64ownload.proto\x12\x17wireless_android_skyjam\"\xf7\x01\n\x18GetTracksToExportRequest\x12\x11\n\tclient_id\x18\x02 \x02(\t\x12\x1a\n\x12\x63ontinuation_token\x18\x03 \x01(\t\x12Y\n\x0b\x65xport_type\x18\x04 \x01(\x0e\x32\x44.wireless_android_skyjam.GetTracksToExportRequest.TracksToExportType\x12\x13\n\x0bupdated_min\x18\x05 \x01(\x03\"<\n\x12TracksToExportType\x12\x07\n\x03\x41LL\x10\x01\x12\x1d\n\x19PURCHASED_AND_PROMOTIONAL\x10\x02\"\xbc\x01\n\x11\x44ownloadTrackInfo\x12\n\n\x02id\x18\x01 \x01(\t\x12\r\n\x05title\x18\x02 \x01(\t\x12\r\n\x05\x61lbum\x18\x03 \x01(\t\x12\x14\n\x0c\x61lbum_artist\x18\x04 \x01(\t\x12\x0e\n\x06\x61rtist\x18\x05 \x01(\t\x12\x14\n\x0ctrack_number\x18\x06 \x01(\x05\x12\x12\n\ntrack_size\x18\x07 \x01(\x03\x12\x13\n\x0b\x64isc_number\x18\x08 \x01(\x05\x12\x18\n\x10total_disc_count\x18\t \x01(\x05\"\x83\x03\n\x19GetTracksToExportResponse\x12W\n\x06status\x18\x01 \x02(\x0e\x32G.wireless_android_skyjam.GetTracksToExportResponse.TracksToExportStatus\x12G\n\x13\x64ownload_track_info\x18\x02 \x03(\x0b\x32*.wireless_android_skyjam.DownloadTrackInfo\x12\x1a\n\x12\x63ontinuation_token\x18\x03 \x01(\t\x12\x13\n\x0bupdated_min\x18\x04 \x01(\x03\"\x92\x01\n\x14TracksToExportStatus\x12\x06\n\x02OK\x10\x01\x12\x13\n\x0fTRANSIENT_ERROR\x10\x02\x12\x1b\n\x17MAX_NUM_CLIENTS_REACHED\x10\x03\x12!\n\x1dUNABLE_TO_AUTHENTICATE_CLIENT\x10\x04\x12\x1d\n\x19UNABLE_TO_REGISTER_CLIENT\x10\x05')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_GETTRACKSTOEXPORTREQUEST_TRACKSTOEXPORTTYPE = _descriptor.EnumDescriptor(
  name='TracksToExportType',
  full_name='wireless_android_skyjam.GetTracksToExportRequest.TracksToExportType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ALL', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PURCHASED_AND_PROMOTIONAL', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=231,
  serialized_end=291,
)
_sym_db.RegisterEnumDescriptor(_GETTRACKSTOEXPORTREQUEST_TRACKSTOEXPORTTYPE)

_GETTRACKSTOEXPORTRESPONSE_TRACKSTOEXPORTSTATUS = _descriptor.EnumDescriptor(
  name='TracksToExportStatus',
  full_name='wireless_android_skyjam.GetTracksToExportResponse.TracksToExportStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OK', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TRANSIENT_ERROR', index=1, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MAX_NUM_CLIENTS_REACHED', index=2, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNABLE_TO_AUTHENTICATE_CLIENT', index=3, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNABLE_TO_REGISTER_CLIENT', index=4, number=5,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=726,
  serialized_end=872,
)
_sym_db.RegisterEnumDescriptor(_GETTRACKSTOEXPORTRESPONSE_TRACKSTOEXPORTSTATUS)


_GETTRACKSTOEXPORTREQUEST = _descriptor.Descriptor(
  name='GetTracksToExportRequest',
  full_name='wireless_android_skyjam.GetTracksToExportRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='client_id', full_name='wireless_android_skyjam.GetTracksToExportRequest.client_id', index=0,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='continuation_token', full_name='wireless_android_skyjam.GetTracksToExportRequest.continuation_token', index=1,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='export_type', full_name='wireless_android_skyjam.GetTracksToExportRequest.export_type', index=2,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='updated_min', full_name='wireless_android_skyjam.GetTracksToExportRequest.updated_min', index=3,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _GETTRACKSTOEXPORTREQUEST_TRACKSTOEXPORTTYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=44,
  serialized_end=291,
)


_DOWNLOADTRACKINFO = _descriptor.Descriptor(
  name='DownloadTrackInfo',
  full_name='wireless_android_skyjam.DownloadTrackInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='wireless_android_skyjam.DownloadTrackInfo.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='title', full_name='wireless_android_skyjam.DownloadTrackInfo.title', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='album', full_name='wireless_android_skyjam.DownloadTrackInfo.album', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='album_artist', full_name='wireless_android_skyjam.DownloadTrackInfo.album_artist', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='artist', full_name='wireless_android_skyjam.DownloadTrackInfo.artist', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='track_number', full_name='wireless_android_skyjam.DownloadTrackInfo.track_number', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='track_size', full_name='wireless_android_skyjam.DownloadTrackInfo.track_size', index=6,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='disc_number', full_name='wireless_android_skyjam.DownloadTrackInfo.disc_number', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total_disc_count', full_name='wireless_android_skyjam.DownloadTrackInfo.total_disc_count', index=8,
      number=9, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=294,
  serialized_end=482,
)


_GETTRACKSTOEXPORTRESPONSE = _descriptor.Descriptor(
  name='GetTracksToExportResponse',
  full_name='wireless_android_skyjam.GetTracksToExportResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='wireless_android_skyjam.GetTracksToExportResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='download_track_info', full_name='wireless_android_skyjam.GetTracksToExportResponse.download_track_info', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='continuation_token', full_name='wireless_android_skyjam.GetTracksToExportResponse.continuation_token', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='updated_min', full_name='wireless_android_skyjam.GetTracksToExportResponse.updated_min', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _GETTRACKSTOEXPORTRESPONSE_TRACKSTOEXPORTSTATUS,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=485,
  serialized_end=872,
)

_GETTRACKSTOEXPORTREQUEST.fields_by_name['export_type'].enum_type = _GETTRACKSTOEXPORTREQUEST_TRACKSTOEXPORTTYPE
_GETTRACKSTOEXPORTREQUEST_TRACKSTOEXPORTTYPE.containing_type = _GETTRACKSTOEXPORTREQUEST
_GETTRACKSTOEXPORTRESPONSE.fields_by_name['status'].enum_type = _GETTRACKSTOEXPORTRESPONSE_TRACKSTOEXPORTSTATUS
_GETTRACKSTOEXPORTRESPONSE.fields_by_name['download_track_info'].message_type = _DOWNLOADTRACKINFO
_GETTRACKSTOEXPORTRESPONSE_TRACKSTOEXPORTSTATUS.containing_type = _GETTRACKSTOEXPORTRESPONSE
DESCRIPTOR.message_types_by_name['GetTracksToExportRequest'] = _GETTRACKSTOEXPORTREQUEST
DESCRIPTOR.message_types_by_name['DownloadTrackInfo'] = _DOWNLOADTRACKINFO
DESCRIPTOR.message_types_by_name['GetTracksToExportResponse'] = _GETTRACKSTOEXPORTRESPONSE

GetTracksToExportRequest = _reflection.GeneratedProtocolMessageType('GetTracksToExportRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETTRACKSTOEXPORTREQUEST,
  __module__ = 'download_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.GetTracksToExportRequest)
  ))
_sym_db.RegisterMessage(GetTracksToExportRequest)

DownloadTrackInfo = _reflection.GeneratedProtocolMessageType('DownloadTrackInfo', (_message.Message,), dict(
  DESCRIPTOR = _DOWNLOADTRACKINFO,
  __module__ = 'download_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.DownloadTrackInfo)
  ))
_sym_db.RegisterMessage(DownloadTrackInfo)

GetTracksToExportResponse = _reflection.GeneratedProtocolMessageType('GetTracksToExportResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETTRACKSTOEXPORTRESPONSE,
  __module__ = 'download_pb2'
  # @@protoc_insertion_point(class_scope:wireless_android_skyjam.GetTracksToExportResponse)
  ))
_sym_db.RegisterMessage(GetTracksToExportResponse)


# @@protoc_insertion_point(module_scope)
